@@ -15,19 +15,21 @@ and efficiently copy-pasting strings in the UI.
 
 Based on splitflap/electronics/scripts/export_util.py by Scott Bezek
 """
-import re
+import argparse
+from contextlib import contextmanager
 import os
-import sys
-from subprocess import Popen, CalledProcessError, TimeoutExpired, call, check_output, STDOUT, DEVNULL, run, PIPE
-import time
+import re
 import shutil
 import signal
+from subprocess import Popen, CalledProcessError, TimeoutExpired, call, check_output, STDOUT, DEVNULL, run, PIPE
+import sys
 from tempfile import mkdtemp
-from contextlib import contextmanager
+import time
 # python3-xvfbwrapper
 from xvfbwrapper import Xvfb
 from kiauto.file_util import get_log_files
-from kiauto.misc import KICAD_VERSION_5_99, MISSING_TOOL
+from kiauto.misc import KICAD_VERSION_5_99, MISSING_TOOL, KICAD_DIED, __version__
+import kiauto.misc
 
 from kiauto import log
 logger = log.get_logger(__name__)
@@ -48,8 +50,9 @@ class PopenContext(Popen):
         # I keep them in case they are needed, but excluded from the coverage.
         # Also note that closing stdin needs extra handling, implemented in the parent class
         # but not here.
-        if self.stdout:
-            self.stdout.close()  # pragma: no cover
+        # This can generate a deadlock
+        # if self.stdout:
+        #     self.stdout.close()  # pragma: no cover
         if self.stderr:
             self.stderr.close()  # pragma: no cover
         if self.stdin:
@@ -99,7 +102,7 @@ def wait_xserver(out_dir, num_try):
         logger.warning('No setxkbmap nor xset available, unable to verify if X is running')
     for i in range(int(timeout/DELAY)):
         logger.debug('Checking using '+str(cmd))
-        flog_out, flog_err = get_log_files(out_dir, cmd[0])
+        flog_out, flog_err, _ = get_log_files(out_dir, cmd[0])
         ret = call(cmd, stdout=flog_out, stderr=flog_err, close_fds=True)
         if not ret:
             time_wait = 0.5*(num_try+1)*(num_try+1)*time_out_scale
@@ -161,7 +164,7 @@ def start_record(do_record, video_dir, video_name):
         cmd = ['recordmydesktop', '--overwrite', '--no-sound', '--no-frame', '--on-the-fly-encoding',
                '-o', video_filename]
         logger.debug('Recording session with: '+str(cmd))
-        flog_out, flog_err = get_log_files(video_dir, cmd[0])
+        flog_out, flog_err, _ = get_log_files(video_dir, cmd[0])
         with PopenContext(cmd, stdout=flog_out, stderr=flog_err, close_fds=True, start_new_session=True) as screencast_proc:
             try:
                 yield
@@ -196,10 +199,10 @@ def start_x11vnc(do_it, old_display):
 
 @contextmanager
 def recorded_xvfb(cfg, num_try=0):
-    try:
-        old_display = os.environ['DISPLAY']
-    except KeyError:
-        old_display = None
+    old_display = os.environ.get('DISPLAY')
+    if cfg.record and shutil.which('recordmydesktop') is None:
+        logger.error('To record the session please install `recordmydesktop`')
+        cfg.record = False
     with Xvfb(width=cfg.rec_width, height=cfg.rec_height, colordepth=cfg.colordepth):
         wait_xserver(cfg.output_dir, num_try)
         with start_x11vnc(cfg.start_x11vnc, old_display):
@@ -212,6 +215,7 @@ def xdotool(command, id=None):
     if id is not None:
         command.insert(1, str(id))
         command.insert(1, '--window')
+    logger.debug(['xdotool'] + command)
     return check_output(['xdotool'] + command, stderr=DEVNULL).decode()
     # return check_output(['xdotool'] + command)
 
@@ -247,7 +251,10 @@ def xdotool(command, id=None):
 
 def text_replace(string):
     """ Used to replace a text in an input text widget. """
-    xdotool(['key', 'ctrl+a', 'type', string])
+    delay = str(int(12*time_out_scale))
+    cmd = ['key', '--delay', delay, 'ctrl+a', 'type', '--delay', delay, string]
+    logger.debug('text_replace with: {}'.format(cmd))
+    xdotool(cmd)
 
 
 def clipboard_retrieve():
@@ -308,7 +315,7 @@ def wait_focused(id, timeout=10):
     timeout *= time_out_scale
     DELAY = 0.5
     logger.debug('Waiting for %s window to get focus...', id)
-    for i in range(int(timeout/DELAY)):
+    for i in range(min(int(timeout/DELAY), 1)):
         cur_id = xdotool(['getwindowfocus']).rstrip()
         logger.debug('Currently focused id: %s', cur_id)
         if cur_id == id:
@@ -445,22 +452,60 @@ def wait_window_change(window_id, x, y, w, h, time_out):
     shutil.rmtree(img_tmp_dir)
 
 
-def open_dialog_with_retry(msg, keys, desc, w_name, cfg):
+def open_dialog_with_retry(msg, keys, desc, w_name, cfg, id_dest=None):
     logger.info(msg)
     wait_point(cfg)
     if cfg.kicad_version >= KICAD_VERSION_5_99:
         # KiCad 6 has a very slow start-up
         time.sleep(1)
-    xdotool(keys)
+    xdotool(keys, id=id_dest)
     retry = False
     try:
-        id = wait_for_window(desc, w_name)
+        id = wait_for_window(desc, w_name, popen_obj=cfg.popen_obj, others=['pcbnew Warning'])
     except RuntimeError:  # pragma: no cover
         # Perhaps the main window wasn't available yet
         retry = True
+    except CalledProcessError as e:
+        logger.error(str(e))
+        sys.exit(KICAD_DIED)
+    except ValueError:
+        logger.warning('Got pcbnew warning dialog, trying to dismiss it')
+        xdotool(['key', 'Return'])
+        retry = True
     if retry:
         logger.info('"{}" did not open, retrying'.format(desc))
         # wait_eeschema_start(cfg)
-        xdotool(keys)
-        id = wait_for_window(desc, w_name)
+        xdotool(keys, id=id_dest)
+        try:
+            id = wait_for_window(desc, w_name, popen_obj=cfg.popen_obj)
+        except CalledProcessError as e:
+            logger.error(str(e))
+            sys.exit(KICAD_DIED)
     return id
+
+
+def show_info():
+    print("This is KiAuto v"+__version__)
+    print("Installed at: "+os.path.abspath(sys.argv[0]))
+    print("Using kiauto module from: "+os.path.dirname(kiauto.misc.__file__))
+    print("Interpreted by Python: {} (v{})".format(sys.executable, sys.version.replace('\n', ' ')))
+    print("Tools:")
+    try:
+        import pcbnew
+        kicad_version = pcbnew.GetBuildVersion()
+        print("- kicad: {} (v{})".format(shutil.which('kicad'), kicad_version))
+    except ImportError:
+        print("ERROR: Failed to import pcbnew Python module."
+              " Is KiCad installed?"
+              " Do you need to add it to PYTHONPATH?")
+    print("- xdotool: "+str(shutil.which('xdotool')))
+    print("- recordmydesktop: "+str(shutil.which('recordmydesktop')))
+    print("- xsltproc: "+str(shutil.which('xsltproc')))
+    print("- xclip: "+str(shutil.which('xclip')))
+    print("- convert: "+str(shutil.which('convert')))
+
+
+class ShowInfoAction(argparse.Action):
+    def __call__(self, parser, namespace, values, option_string=None):
+        show_info()
+        exit(0)
